package version;

import java.io.File;
import java.io.FilenameFilter;
import java.io.IOException;
import java.net.URL;
import java.net.URLClassLoader;
import java.util.jar.JarFile;
import java.util.jar.Manifest;

public class VersionClassLoader extends URLClassLoader {
	public static final int VERSION_EQ = 1;
	public static final int VERSION_NE = 2;
	public static final int VERSION_GT = 3;
	public static final int VERSION_GE = 4;
	public static final int VERSION_LT = 5;
	public static final int VERSION_LE = 6;
	private static long s2l(String s, int pad) {
		long res = 0;
		for(String part : s.split("\\.")) {
			res = 1000 * res + Integer.parseInt(part);
		}
		for(int i = 0; i < pad; i++) {
			res *= 100;
		}
		return res;
	}
	private static boolean match(String ver1, int compare, String ver2) {
		int n1 = ver1.split("\\.").length;
		int n2 = ver1.split("\\.").length;
		int nmax = Math.max(n1, n2);
		long v1 = s2l(ver1, nmax - n1);
		long v2 = s2l(ver2, nmax - n2);
		switch(compare) {
			case VERSION_EQ:
				return v1 == v2;
			case VERSION_NE:
				return v1 != v2;
			case VERSION_GT:
				return v1 > v2;
			case VERSION_GE:
				return v1 >= v2;
			case VERSION_LT:
				return v1 < v2;
			case VERSION_LE:
				return v1 <= v2;
			default:
				return false;
		}
	}
	private static URL[] getUrls(String dir, String name, int compare, String reqver) {
		File dirobj = new File(dir);
		FilenameFilter filt = new FilenameFilter() {
			@Override
			public boolean accept(File dir, String name) {
				return name.endsWith(".jar");
			}
		};
		for(String fnm : dirobj.list(filt)) {
	        try {
	        	String fullfnm = dir + System.getProperty("file.separator") + fnm;
				JarFile jar = new JarFile(fullfnm);
				Manifest mf = jar.getManifest();
				String ver = mf.getEntries().get(name).getValue("Implementation-Version");
				if(match(reqver, compare, ver)) {
					File jarobj = new File(fullfnm);
					return new URL[] { jarobj.toURI().toURL() };
				}
			} catch (IOException e) {
				return new URL[0];
			}
		}
		return new URL[0];
	}
	public VersionClassLoader(String dir, String name, int compare, String version) {
		super(getUrls(dir, name, compare, version));
	}
	public VersionClassLoader(String dir, String name, String version) {
		this(dir, name, VERSION_EQ, version);
	}
}
